/*
 * Interface summary generation.
 *
 * Copyright (C) 2019, 2022 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <stdio.h>
#include "summary.h"



static void indent(int level)
{
  while (level--) putchar(' ');
}

static void show_imports(struct import *imp, int level)
{
  if (imp == NULL)
    return;

  show_imports(imp->tail, level);

  indent(level);
  printf("%s: %s\n", "Import", imp->filename);
}

static void show_includes(struct include *inc, int level)
{
  if (inc == NULL)
    return;

  show_includes(inc->tail, level);

  indent(level);
  printf("%s: %s\n", "Include", inc->filename);
}

static void show_identifiers(struct identifier *ident)
{
  while (ident != NULL)
  {
    printf(" %s", ident->identifier);
    ident = ident->tail;
  }
}

static void show_parameter(struct parameter *param, int level)
{
  char *type;

  indent(level);
  printf("Parameter: %s%s", param->specifier & IN_PARAMETER ? "IN" : "",
                            param->specifier & OUT_PARAMETER ? "OUT" : "");

  /* Items must have their type identifiers restored. */

  if (param->cls & ITEM_CLASS)
  {
    type = param->cls == FPAGE_ITEM ? "fpage" : "cap";
    printf(" %s", type);
  }

  show_identifiers(param->identifiers);
  printf("\n");
}

static void show_parameters(struct parameter *param, int level)
{
  if (param == NULL)
    return;

  show_parameter(param, level);
  show_parameters(param->tail, level);
}

static void show_attribute(struct attribute *attr, int level)
{
  indent(level);
  printf("Attribute: %s", attr->attribute);

  show_identifiers(attr->identifiers);
  printf("\n");
}

static void show_attributes(struct attribute *attr, int level)
{
  if (attr == NULL)
    return;

  show_attribute(attr, level);
  show_attributes(attr->tail, level);
}

static void show_signature(struct signature *sig, int level)
{
  indent(level);
  printf("Signature: %s %s\n", sig->qualifier, sig->operation);

  show_attributes(sig->attributes, level + 2);
  show_parameters(sig->parameters, level + 2);
}

static void show_signatures(struct signature *sig, int level)
{
  if (sig == NULL)
    return;

  show_signature(sig, level);
  show_signatures(sig->tail, level);
}

static void show_bases(struct interface_ref *base, int level)
{
  if (base == NULL)
    return;

  indent(level);
  printf("Base: %s\n", base->name);

  if (base->iface != NULL)
    show_interface(base->iface, level + 2);
  show_bases(base->tail, level);
}



/* Public functions. */

void show_interface(struct interface *iface, int level)
{
  indent(level);
  printf("Interface: %s\n", iface->name);
  indent(level);
  printf("Basename: %s\n", iface->output_basename);

  show_bases(iface->bases, level + 2);
  show_imports(iface->imports, level);
  show_includes(iface->includes, level);
  show_attributes(iface->attributes, level);
  show_signatures(iface->signatures, level);
}

void show_interfaces(struct interface *iface)
{
  if (iface->tail != NULL)
    show_interfaces(iface->tail);

  show_interface(iface, 0);
}
